#ifndef NBOOTP_H
#define NBOOTP_H

#ifdef __cplusplus
extern "C" {
#endif

#define BOOTREQUEST     1
#define BOOTREPLY       2
#define BOOTANNOUNCE    3
#define ARPOP_REQUEST   1		
#define ARPOP_REPLY	    2	

/* h/w type */
/* http://www.freesoft.org/CIE/RFC/951/3.htm *
 * http://www.faqs.org/rfcs/rfc1700.html */
typedef enum
{
    ETH_10=1,      /* 10mb ethernet */
}nbootp_hwtype;

/* values for use in ARP when looking for a reply */
typedef enum
{
    ARP_NO_REPLY,
    ARP_RXD_REPLY
} arp_reply;

/* MAC add length */
#define HW_ADD_LEN 6 /* IEEE 802 MAC addresses */
/* Address Resolution packet	*/
#define ETH_P_ARP	0x0806	
/* Internet Protocol packet	*/
#define ETH_P_IP	0x0800		
/* IP packet, UDP protocol */
#define IP_P_UDP   0x11
#define IP_P_VERS  4
#define IP_P_TTL   64
/* ARP add length */
#define ARP_ADD_LEN 4
/* Server port */
#define SEND_PORT 5125 /* send port */
/* Client port */
#define RCV_PORT 5126 /* receive port */
/* Destination IP */
#define BROADCAST_IP "255.255.255.255" /*broadcast IP */
/* Module Netmask */
#define NETMASK "255.0.0.0"; /* netmask */
/* wait time for BootP response */
#define BOOTP_WAIT      60 /* 1 minute */
/* wait time for ARP response */
#define ARP_WAIT        2 /* 2 sec */
#define NUM_ARP_RETRIES 2
#define ARP_CONFLICT_RETRY_TIMEOUT 15 /* 15 seconds */

#define NUM_OPTION_BYTES    60
#define NUM_PADDING_BYTES   248
#define MAC_BROADCAST_NIBBLE    0xff  

struct ip_header_t 
{
    /* we are big endian, should we just hard code to that or leave in the conditional */
#if __BYTE_ORDER == __LITTLE_ENDIAN
    unsigned int ihl:4;
    unsigned int version:4;
#elif __BYTE_ORDER == __BIG_ENDIAN
    unsigned int version:4;
    unsigned int ihl:4;
#else
# error "Please fix <bits/endian.h>"
#endif
    uint8_t tos;
    uint16_t tot_len;
    uint16_t id;
    uint16_t frag_off;
    uint8_t ttl;
    uint8_t protocol;
    uint16_t check;
    uint32_t saddr;
    uint32_t daddr;
} __attribute__ ((__packed__));

struct udp_header_t
{
    uint16_t source;
    uint16_t dest;
    uint16_t len;
    uint16_t check;
} __attribute__ ((__packed__));

struct bootp_data_t
{
    uint8_t op;
    uint8_t htype;
    uint8_t hlen;
    uint8_t hops;
    uint32_t xid;
    uint16_t secs;
    uint16_t flags;
    uint32_t ciaddr;
    uint32_t yiaddr;
    uint32_t siaddr;
    uint32_t giaddr;
    uint8_t chaddr[16];
    uint8_t sname[64];
    uint8_t file[128];
    uint32_t cookie;
    uint8_t options[60]; 
} __attribute__ ((__packed__));

struct bootp_packet_t 
{
    struct ip_header_t ip_hdr;
    struct udp_header_t udp_hdr;
    struct bootp_data_t data;
    uint8_t padding[248];
} __attribute__ ((__packed__));

/* IB NetApp vendor-specific area */
struct bootp_ib_vendor_data_t
{
    uint8_t hw_type;
    uint8_t hw_ver;
    uint8_t fw_ver_major;
    uint8_t fw_ver_minor;
    uint8_t timestamp[14];
    uint8_t reserved[46];
} __attribute__ ((__packed__));

/* Input optons */
struct options_t
{
    int announce;
    int retry;
    int flags;
    int hw_type;
    int  hw_ver;
    int fw_ver;
    char req_ip[32];
    uint32_t s_ip;
    char s_id[64];
    int unused_flag;
    char *interface;  
} __attribute__ ((__packed__));

struct arp_pkt_t
{
    /* Ethernet header */
	uint8_t  h_dest[6];     /* destination ether addr */
	uint8_t  h_source[6];   /* source ether addr */
	uint16_t h_proto;       /* packet type ID field */

	/* ARP packet */
	uint16_t htype;         /* hardware type (must be ARPHRD_ETHER) */
	uint16_t ptype;         /* protocol type (must be ETH_P_IP) */
	uint8_t  hlen;          /* hardware address length (must be 6) */
	uint8_t  plen;          /* protocol address length (must be 4) */
	uint16_t operation;     /* ARP opcode */
	uint8_t  sHaddr[6];     /* sender's hardware address */
	uint32_t  sInaddr;      /* sender's IP address */
	uint8_t  tHaddr[6];     /* target's hardware address */
	uint32_t  tInaddr;      /* target's IP address */
	uint8_t  pad[18];       /* pad for min. ethernet payload (60 bytes) */
} __attribute__ ((__packed__));

#ifdef __cplusplus
}
#endif

#endif /* NBOOTP_H */
