--- old/drivers/mtd/spichips/generic.c	2013-12-05 13:39:25.855446000 -0500
+++ new/drivers/mtd/spichips/generic.c	2013-12-06 10:13:55.066658000 -0500
@@ -21,6 +21,10 @@
 #include <common.h>
 #endif
 #include "spiflash.h"
+#include <linux/module.h>
+#include <linux/proc_fs.h>
+#include <linux/seq_file.h>
+
 #ifdef	CFG_FLASH_SPI_DRIVER
 
 
@@ -47,6 +51,20 @@
 
 #define PROGRAM_PAGE_SIZE	256	/* Max Program Size */
 
+static int identspiflash_open(struct inode *inode, struct  file *file);
+
+#define MAX_FLASHNAME_LEN       128
+char spi_chipname[MAX_FLASHNAME_LEN] = {0};
+
+static const struct file_operations identspiflash_fops = {
+  .owner = THIS_MODULE,
+  .open = identspiflash_open,
+  .read = seq_read,
+  .llseek = seq_lseek,
+  .release = single_release,
+};
+
+
 static 
 int inline 
 spi_error(int retval)
@@ -333,6 +351,8 @@
 	int i;
 	u8 code = OPCODE_RDID;
 
+        spi_chipname[0] = 0;
+        
 	if (spi_verbose == 2) 
 		printk("SPI: probing for %s devices ...\n",spi_name);
 
@@ -375,6 +395,8 @@
 	}
 	memcpy(chip_info,&spi_list[i],sizeof(struct spi_flash_info));
 
+        snprintf(spi_chipname, MAX_FLASHNAME_LEN, "Found SPI Chip %s \n", spi_list[i].name);
+
 	if (spi_verbose > 0)
 		printk(KERN_INFO"Found SPI Chip %s \n",spi_list[i].name);
 
@@ -382,6 +404,27 @@
 
 }
 
+static int identspiflash_show(struct seq_file *m, void *v) {
+  seq_printf(m, spi_chipname);
+  return 0;
+}
+
+static int identspiflash_open(struct inode *inode, struct  file *file) {
+  return single_open(file, identspiflash_show, NULL);
+}
+
+static void __exit spi_generic_chip_exit(void) {
+  remove_proc_entry("spiflash", NULL);
+}
+
+static int __init spi_generic_chip_init(void) {
+
+  proc_create("spiflash", 0, NULL, &identspiflash_fops);
+
+  return 0;
+}
+
+
 EXPORT_SYMBOL(spi_generic_probe);
 EXPORT_SYMBOL(spi_generic_erase);
 EXPORT_SYMBOL(spi_generic_read);
@@ -392,6 +435,9 @@
 EXPORT_SYMBOL(spi_generic_write_status);
 EXPORT_SYMBOL(spi_generic_wait_till_ready);
 
+module_init(spi_generic_chip_init);
+module_exit(spi_generic_chip_exit);
+
 MODULE_LICENSE("GPL");
 MODULE_AUTHOR("American Megatrends Inc");
 MODULE_DESCRIPTION("MTD SPI driver for Generic SPI flash chips");
